/*  
    whitespc.c written by The Last Viking, paal@ekran.no
    (C) 4.4.97 Ekran Design.

    This program reads a text-file, then it searches for spaces, i.e. ' '
    at the end of the line, before the linefeed character. If if finds any,
    it removes them.

    To compile this program you need a C compiler. If you, for instance
    have gcc, you should write:
  
    gcc -O -owhitespc whitespc.c
*/

#include <stdio.h>

struct record {                           /* structure used to hold the file */
   char ch;
   struct record *prev;
   struct record *next;
};

char *infile[80];                             /* filename for the input file */
char *outfile[80];                           /* filename for the output file */
char ch;
FILE *fp;                                                     /* filepointer */

struct record *rhead,*rlist,*rprev;               /* pointers to the buffer */

void main(int argc,char **argv) {
   int x,y;
   int chk,wspc;
   int inarg,outarg;
   int old,new;

   inarg=1;outarg=1; chk=0; old=0; new=0;

   /* let's spend the next megabyte pasing the input arguments */
 
   if ((argc<2) || ((argc==2) && argv[1][0]=='-')) {
     printf("Usage:\n");
     printf("%s%s%s","  ",argv[0]," [-l] [-o file] [file]\n");
     printf("\nWhiteSPC strips space-characters from the end of text lines.\n");
     printf("-l warn if a line is longer than 80 characters.\n");
     printf("-o output file [write to input file if not specified].\n");
     exit(0);
   }
 
   if ((argc==3) && ((argv[1][0]=='-') && (argv[1][1]=='l'))) chk=1;
   if ((argc==3) && ((argv[2][0]=='-') && (argv[2][1]=='l'))) chk=1;

   if ((argc==4) && ((argv[1][0]=='-') && (argv[1][1]=='l'))) chk=1;
   if ((argc==4) && ((argv[2][0]=='-') && (argv[2][1]=='l'))) chk=1;
   if ((argc==4) && ((argv[3][0]=='-') && (argv[3][1]=='l'))) chk=1;
  
   if ((argc==2) && (argv[1][0]!='-')) { /* only one parameter which is file */
    inarg=1; outarg=1;
   }

   if ((argc==3) && (((argv[1][0]=='-') || (argv[2][0]=='-')) &&
                     ((argv[1][1]=='l') || (argv[2][1]=='l'))))
                     if (argv[1][0]=='-') {inarg=2; outarg=2;}
                     else {inarg=1; outarg=1;}
   
   if ((argc==3) && (((argv[1][0]=='-') || (argv[2][0]=='-')) &&
                     ((argv[1][1]=='o') || (argv[2][1]=='o'))))
                     if (argv[1][0]=='-') {inarg=2; outarg=1;}
                     else {inarg=1; outarg=2;}
   
   if (argc==4) {
      if ((argv[1][0]=='-') && (argv[1][1]=='o')) outarg=1;
      if ((argv[2][0]=='-') && (argv[2][1]=='o')) outarg=2;
      if ((argv[3][0]=='-') && (argv[3][1]=='o')) outarg=3;
      if (argv[1][0]!='-') inarg=1;
      if (argv[2][0]!='-') inarg=2;
      if (argv[3][0]!='-') inarg=3;
   }

   printf("WhiteSpc v1.3 by The Last Viking, paal@ekran.no (C) 97 Ekran Design.\n\n");
 
   if (!(fp=fopen(argv[inarg],"r"))) {  /* open file */
    printf("can't open input file.\n");exit(0);
   }

   rhead=(struct record*) malloc(sizeof(struct record));/* create empty head */
   rhead->ch='!'; rhead->next=NULL; rhead->prev=NULL;
   rprev=rhead;
   while (ch!=EOF) {                               /* loop until end of file */
      ch=fgetc(fp)&0xff;                                   /* next character */
      if (ch!=EOF) {
         rlist=(struct record*) malloc(sizeof(struct record)); /* next entry */
	     rlist->ch=ch; rlist->next=NULL; rlist->prev=rprev;
         rprev->next=rlist;
         rprev=rlist; 
     }
     old++;
   }
   fclose(fp);
   x=0;y=1; 
   rlist=rhead->next;                         /* remove SPC before the LF's */
   while (rlist) {
      while ((rlist->ch!=0xa) && (rlist->next!=NULL)) {
        rlist=rlist->next;
        x++;
      }
      if ((chk==1) && (x>80)) 
         printf("%s%d%s%d%s","Line: ",y," is ",x," characters wide.\n");
      y++; x=0;
      while (rlist->prev->ch==' ') {                  /* found spc before lf */
        rlist->prev->prev->next=rlist;/* point the one before spc to current */
        rlist->prev=rlist->prev->prev; /* point currents prev to prev's prev */
      }
      rlist=rlist->next;                                       /* next entry */
   } 
   
   if (outarg==inarg) {
      if (!(fp=fopen(argv[outarg],"w"))) {printf("can't open output file.\n");
         exit(0);
      }
   }
   else {
      if (argv[outarg][2]==' ') wspc=3; else wspc=2;
      if (!(fp=fopen(argv[outarg]+wspc,"w"))) {
         printf("can't open output file.\n");
         exit(0);
      }
   }
   rlist=rhead->next;                        /* remove header and print file */
   while (rlist) {
      new++;
      fprintf(fp,"%c",rlist->ch);
      rlist=rlist->next;
   } 
   fclose(fp);                                         /* close output file */
  
   printf("%s%s%s%d%s\n","old filename: ",argv[inarg]," - ",old-1," bytes.");
   if (outarg==inarg) 
     printf("%s%s%s%d%s\n","new filename: ",argv[outarg]," - ",new," bytes.");
    else 
     printf("%s%s%s%d%s\n","new filename: ",argv[outarg]+wspc," - ",new," bytes.");
}

